package com.ustadmobile.core.db.dao

import com.ustadmobile.door.DoorDatabase
import com.ustadmobile.door.ext.doorPrimaryKeyManager
import com.ustadmobile.door.ext.rootDatabase
import com.ustadmobile.door.util.systemTimeInMillis
import com.ustadmobile.lib.db.entities.ContentEntry
import com.ustadmobile.lib.db.entities.ContentEntryParentChildJoin
import kotlin.Long
import kotlin.Unit
import kotlin.collections.List

public class ContentEntryParentChildJoinDao_ReplicateWrapper(
  private val _db: DoorDatabase,
  private val _dao: ContentEntryParentChildJoinDao
) : ContentEntryParentChildJoinDao() {
  public override suspend fun replicateOnNewNode(newNodeId: Long): Unit {
    _dao.replicateOnNewNode(newNodeId)
  }

  public override suspend fun replicateOnChange(): Unit {
    _dao.replicateOnChange()
  }

  public override suspend fun insertListAsync(entityList: List<ContentEntryParentChildJoin>): Unit {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    val _generatedPks = mutableListOf<Long>()
    entityList.iterator().forEach  {
      if(it.cepcjUid == 0L) {
        val _newPk = _pkManager.nextIdAsync(7)
        it.cepcjUid = _newPk
        _generatedPks += _newPk
      }
      it.cepcjLct = systemTimeInMillis()
    }
    //must set versionid and/or primary key here
    _dao.insertListAsync(entityList)
  }

  public override fun publicContentEntryParentChildJoins(): List<ContentEntryParentChildJoin> =
      _dao.publicContentEntryParentChildJoins()

  public override fun all(): List<ContentEntryParentChildJoin> = _dao.all()

  public override fun findParentByChildUuids(childEntryContentUid: Long):
      ContentEntryParentChildJoin? = _dao.findParentByChildUuids(childEntryContentUid)

  public override fun findListOfParentsByChildUuid(childEntryContentUid: Long):
      List<ContentEntryParentChildJoin> = _dao.findListOfParentsByChildUuid(childEntryContentUid)

  public override fun findListOfChildsByParentUuid(parentUid: Long):
      List<ContentEntryParentChildJoin> = _dao.findListOfChildsByParentUuid(parentUid)

  public override fun findJoinByParentChildUuids(parentUid: Long, childUid: Long):
      ContentEntryParentChildJoin? = _dao.findJoinByParentChildUuids(parentUid, childUid)

  public override suspend fun selectTopEntries(): List<ContentEntry> = _dao.selectTopEntries()

  public override fun replaceList(entries: List<ContentEntryParentChildJoin>): Unit {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    val _generatedPks = mutableListOf<Long>()
    entries.iterator().forEach  {
      if(it.cepcjUid == 0L) {
        val _newPk = _pkManager.nextId(7)
        it.cepcjUid = _newPk
        _generatedPks += _newPk
      }
      it.cepcjLct = systemTimeInMillis()
    }
    //must set versionid and/or primary key here
    _dao.replaceList(entries)
  }

  public override fun insertWithReplace(parentChildJoinDao: ContentEntryParentChildJoin): Unit {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    if(parentChildJoinDao.cepcjUid == 0L) {
      val _newPk = _pkManager.nextId(7)
      parentChildJoinDao.cepcjUid = _newPk
    }
    parentChildJoinDao.cepcjLct = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.insertWithReplace(parentChildJoinDao)
  }

  public override suspend fun moveListOfEntriesToNewParent(
    contentEntryUid: Long,
    selectedItems: List<Long>,
    updateTime: Long
  ): Unit {
    _dao.moveListOfEntriesToNewParent(contentEntryUid, selectedItems, updateTime)
  }

  public override fun insert(entity: ContentEntryParentChildJoin): Long {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    if(entity.cepcjUid == 0L) {
      val _newPk = _pkManager.nextId(7)
      entity.cepcjUid = _newPk
    }
    entity.cepcjLct = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.insert(entity)
    return entity.cepcjUid
  }

  public override suspend fun insertAsync(entity: ContentEntryParentChildJoin): Long {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    if(entity.cepcjUid == 0L) {
      val _newPk = _pkManager.nextIdAsync(7)
      entity.cepcjUid = _newPk
    }
    entity.cepcjLct = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.insertAsync(entity)
    return entity.cepcjUid
  }

  public override fun insertList(entityList: List<out ContentEntryParentChildJoin>): Unit {
    val _pkManager = _db.rootDatabase.doorPrimaryKeyManager
    val _generatedPks = mutableListOf<Long>()
    entityList.iterator().forEach  {
      if(it.cepcjUid == 0L) {
        val _newPk = _pkManager.nextId(7)
        it.cepcjUid = _newPk
        _generatedPks += _newPk
      }
      it.cepcjLct = systemTimeInMillis()
    }
    //must set versionid and/or primary key here
    _dao.insertList(entityList)
  }

  public override fun updateList(entityList: List<out ContentEntryParentChildJoin>): Unit {
    val _generatedPks = mutableListOf<Long>()
    entityList.iterator().forEach  {
      it.cepcjLct = systemTimeInMillis()
    }
    //must set versionid and/or primary key here
    _dao.updateList(entityList)
  }

  public override fun update(entity: ContentEntryParentChildJoin): Unit {
    entity.cepcjLct = systemTimeInMillis()
    //must set versionid and/or primary key here
    _dao.update(entity)
  }
}
